# -*- coding: utf-8 -*-

import json
import numpy as np

class NumpyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

class SILM_Config:
    cfg = None
    angle = None
    pitch = None
    phase = None
    depth = None
    power = None
            
    # Convert to 6x4 array used by the fitting code
    def get_mod(self):
        mod = np.zeros((len(self.phase),5))
        freq = 2*np.pi / self.pitch
        ang = np.deg2rad(self.angle)
        mod[:,0] = freq * np.cos(ang)
        mod[:,1] = freq * np.sin(ang)
        mod[:,2] = self.depth
        mod[:,3] = self.phase
        mod[:,4] = self.power
        return mod
    
    def __init__(self, cfg, angle, pitch, depth, phase, power):
        self.cfg = cfg
        self.angle = angle
        self.pitch = pitch
        self.depth = depth
        self.phase = phase
        self.power = power
        
    def save(self, filename):
        with open(filename, "w")  as f:
            json.dump(self.__dict__,f,indent="\t",cls=NumpyEncoder) 
            
            
def from_mod(cfg, mod) -> SILM_Config:
    kx = mod[:,0]
    ky = mod[:,1]
    freq = np.sqrt(kx**2+ky**2)
    return SILM_Config(cfg, 
                angle= np.rad2deg( np.arctan2(ky,kx) ),
                pitch =2*np.pi/freq,
                depth = mod[:,2],
                phase = mod[:,3],
                power = mod[:,4])
    
    
def load(filename) -> SILM_Config:
    with open(filename, "r") as f:
        data = json.load(f)
        return SILM_Config(data['cfg'],data['angle'], data['pitch'], data['depth'], data['phase'], data['power'])

