# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

import ctypes
from .base import SMLM
import numpy as np
import numpy.ctypeslib as ctl


class PostProcess:
    def __init__(self, smlmlib):
        self.lib = smlmlib

        # CDLL_EXPORT void NearestNeighborDriftEstimate(const Vector3f * xyI, const int *spotFrameNum,
        # int numspots, float searchDist, const Int2 *framePairs, int numFramePairs, Vector2f *drift,
        # int *matchResults, int width, int height, int icpIterations)

        self._NearestNeighborDriftEstimate = smlmlib.lib.NearestNeighborDriftEstimate
        self._NearestNeighborDriftEstimate.argtypes = [
            ctl.ndpointer(np.float32, flags="aligned, c_contiguous"),  # xyI Vector3f[]
            ctl.ndpointer(np.int32, flags="aligned, c_contiguous"),  # indices int[]
            ctypes.c_int32,  # numspots
            ctypes.c_float,  # searchDist
            ctl.ndpointer(np.int32, flags="aligned, c_contiguous"),  # framepairs int2[]
            ctypes.c_int32,  # numframepairs
            ctl.ndpointer(np.float32, flags="aligned, c_contiguous"),  # drift Vector2f[numframepairs]
            ctl.ndpointer(np.int32, flags="aligned, c_contiguous"),  # match results int[numframepairs]
            ctypes.c_int32,  # width
            ctypes.c_int32,  # height
            ctypes.c_int32,  # icpIterations
        ]

    def NearestNeighborDriftEstimate(self, xyI, indices, framepairs, imgshape, searchDist=2, icpIterations=4):
        xyI = np.ascontiguousarray(xyI, dtype=np.float32)
        numspots = xyI.shape[0]

        if xyI.shape[1] == 2:
            xyI_ = np.zeros((len(xyI), 3), dtype=np.float32)
            xyI_[:, 0:2] = xyI
            xyI = xyI_

        assert xyI.shape[1] == 3
        assert framepairs.shape[1] == 2

        indices = np.ascontiguousarray(indices, dtype=np.int32)

        framepairs = np.ascontiguousarray(framepairs, dtype=np.int32)
        numFramePairs = len(framepairs)
        drift = np.zeros((numFramePairs, 2), dtype=np.float32)
        matchResults = np.zeros((numFramePairs), dtype=np.int32)

        self._NearestNeighborDriftEstimate(
            xyI,
            indices,
            numspots,
            searchDist,
            framepairs,
            numFramePairs,
            drift,
            matchResults,
            imgshape[1],
            imgshape[0],
            icpIterations,
        )

        return drift, matchResults
