# -*- coding: utf-8 -*-
import os
import pickle
import numpy as np

# Call func(path, cfg) only if the cached version has a different value of 'cfg'.
# If equal, return the cached data
def read(path, cache_tag, cfg, func, rebuild=False):
    dir, file = os.path.split(path)
    file, ext = os.path.splitext(file)

    path_noext, _ = os.path.splitext(path)

    configfn = path_noext + f".{cache_tag}.cfg.pickle"
    resultsfn = path_noext + f".{cache_tag}.pickle"
    if not rebuild and os.path.exists(configfn):
        with open(configfn, "rb") as fcfg:
            stored_cfg = pickle.load(fcfg)
            if stored_cfg == cfg:
                print(f"Using cached {resultsfn}")
                with open(resultsfn, "rb") as f:
                    return pickle.load(f)

    r = func(path, cfg)

    with open(configfn, "wb") as f:
        pickle.dump(cfg, f)
    with open(resultsfn, "wb") as f:
        pickle.dump(r, f)

    return r


# Using numpy save/load. Pickles above 4GB fail, so this is a way around. func() is supposed to return a tuple of numpy arrays
def read_npy(path, cache_tag, cfg, func, rebuild=False, report=False):
    dir, file = os.path.split(path)
    file, ext = os.path.splitext(file)

    path_noext, _ = os.path.splitext(path)

    resultfn = path_noext + f".{cache_tag}.npy"
    configfn = path_noext + f".{cache_tag}.cfg.pickle"
    if not rebuild and os.path.exists(configfn):
        with open(configfn, "rb") as fcfg:
            stored_cfg, numarrays = pickle.load(fcfg)
            if stored_cfg == cfg:
                # print(f'Using cached {resultsfn}')

                r = [0] * numarrays
                with open(resultfn, "rb") as f:
                    for k in range(numarrays):
                        r[k] = np.load(f)
                return tuple(r)

    r = func(path, cfg)

    with open(resultfn, "wb") as f:
        for k in range(len(r)):
            if report:
                print(f"{resultfn} Saving array {k} ({r[k].shape}, {r[k].dtype})")
            np.save(f, r[k])

    with open(configfn, "wb") as f:
        pickle.dump((cfg, len(r)), f)

    return r
