import matplotlib.pyplot as plt
import numpy as np, sys
import utils.lsqfit as lsqfit
from . import localizations as loc

from gaussian.fitters import fit_sigma_2d
from smlmlib.base import SMLM
import tqdm


# ft( ft(in1) * conj(ft(in2)) );
def crosscorrelation(A, B, smlm: SMLM):
    A_fft = np.fft.fft2(A)
    B_fft = np.fft.fft2(B)
    return np.fft.ifft2(A_fft * np.conj(B_fft))


def crosscorrelation_cuda(A, B, smlm: SMLM):
    return smlm.IFFT2(np.conj(smlm.FFT2(A)) * smlm.FFT2(B))


def estimate(xyI, framenum, imgshape, smlm, scale=4, maxdrift=4):
    numframes = int(np.max(framenum) + 1)
    shape = (imgshape[0] * scale, imgshape[1] * scale)

    registrationFrame = 0  # numframes//2

    # print(f'Frame0: {np.sum(framenum==0)}')
    img0 = loc.draw_spots(
        np.zeros(shape),
        xyI[framenum == registrationFrame, 0] * scale,
        xyI[framenum == registrationFrame, 1] * scale,
        1,
        1,
        1,
        1,
        smlm,
    )

    drift = np.zeros((numframes - 1, 2))

    # split up in bins to make the cuda crosscorrelation go faster
    for f in np.arange(numframes - 1):
        ind = framenum == f + 1
        img = loc.draw_spots(
            np.zeros(shape, dtype=np.float32), xyI[ind, 0] * scale, xyI[ind, 1] * scale, xyI[ind, 2], 1, 1, 1, smlm
        )

        sys.stdout.write(f"\rdrift estimating {f}/{numframes-1}")

        r = 6  # fit window is [r*2, r*2]

        #       cc = crosscorrelation_cuda( np.tile(img0,(len(img),1,1)), img, smlm)
        cc = crosscorrelation_cuda(img0, img, smlm)
        cc = np.abs(np.fft.fftshift(cc, (-2, -1)))

        # look for the peak in a small subsections

        hw = int(maxdrift * scale)
        cc_middle = cc[cc.shape[0] // 2 - hw : cc.shape[0] // 2 + hw, cc.shape[1] // 2 - hw : cc.shape[1] // 2 + hw]
        peak = np.array(np.unravel_index(np.argmax(cc_middle), cc_middle.shape))
        peak += [cc.shape[0] // 2 - hw, cc.shape[1] // 2 - hw]

        peak = np.clip(peak, r, np.array(shape) - r)
        roi = cc[peak[0] - r + 1 : peak[0] + r, peak[1] - r + 1 : peak[1] + r]
        if False:
            plt.figure()
            plt.imshow(roi)
            plt.figure()
            plt.plot(roi[r])
            coeff = lsqfit.lsqfit(roi)
            Y, X = np.indices(roi.shape)
            plt.plot(lsqfit.lsqeval(Y, X, coeff)[r])

        roi_top = fit_sigma_2d(roi, initial_sigma=2)[[0, 1]]
        #            roi_top = lsqfit.lsqfitmax(roi)
        top = roi_top + np.array([peak[1], peak[0]])
        drift[f, 0] = -(top[0] - r + 1 - shape[1] / 2) / scale
        drift[f, 1] = -(top[1] - r + 1 - shape[0] / 2) / scale

    sys.stdout.write("\n")
    return drift
