#include "MemLeakDebug.h"
#include "CameraCalibration.h"


sCMOS_CalibrationTransform::sCMOS_CalibrationTransform(const ImageQueueConfig & iqc)
{
	int2 s = iqc.ImageSize();
	gain.Init(s);
	vargain2.Init(s);
	offset.Init(s);
}

void sCMOS_CalibrationTransform::ProcessImage(DeviceImage<float>& sample,cudaStream_t stream)
{
	auto smp = sample.GetIndexer();
	auto offset = this->offset.GetIndexer();
	auto gain = this->gain.GetIndexer();
	auto vargain2 = this->vargain2.GetIndexer();

	LaunchKernel(sample.width, sample.height, [=]__device__(int x, int y) {
		smp.pixel(x,y) = (smp.pixel(x,y) - offset.pixel(x, y)) / gain.pixel(x, y) + vargain2.pixel(x, y);
	}, 0, stream);
}

void sCMOS_CalibrationTransform::Completed()
{
}



SampleOffset_sCMOS<float> sCMOS_CalibrationTransform::GetSampleOffset()
{
	SampleOffset_sCMOS<float> offset;

	offset.d_vargain2 = vargain2.data;
	return offset;
}

void sCMOS_CalibrationTransform::SetCalibrationImages(const float * h_offset, const float * h_gain, const float * h_variance)
{
	float* h_vargain2 = new float[vargain2.NumPixels()];
	for (int i = 0; i < vargain2.NumPixels(); i++)
		h_vargain2[i] = h_variance[i] / (h_gain[i] * h_gain[i]);

	vargain2.CopyFromHost(h_vargain2);
	delete[] h_vargain2;

	offset.CopyFromHost(h_offset);
	gain.CopyFromHost(h_gain);
}

CDLL_EXPORT void sCMOS_SetCameraCalibration(sCMOS_CalibrationTransform * task, const float * offset, const float * gain, const float * variance)
{
	task->SetCalibrationImages(offset, gain, variance);
}
