// sCMOS camera calibration implemented based on 
// Video-rate nanoscopy using sCMOS cameraspecific single-molecule localization algorithms
// https://www.nature.com/articles/nmeth.2488
#pragma once

#include <vector>
#include "ImageProcessingQueue.h"
#include "Estimation.h"

class sCMOS_CalibrationTransform : public IImageProcessor
{
public:
	sCMOS_CalibrationTransform(const ImageQueueConfig & iqc);

	void ProcessImage(DeviceImage<float>& img, cudaStream_t stream);
	void Completed();
	SampleOffset_sCMOS<float> GetSampleOffset();

	void SetCalibrationImages(const float* offset, const float* gain, const float * variance);

protected:
	struct StreamData {
		DeviceImage<float> corrected;
	};
	std::vector<StreamData> streamData;
	IImageSource* imageSource;

	DeviceImage<float> offset, gain, vargain2;
};

CDLL_EXPORT void sCMOS_SetCameraCalibration(sCMOS_CalibrationTransform* task, const float* offset, const float* gain, const float *variance);

