#pragma once

#include "SILM_Structs.h"
#include "palala.h"
#include <cuda_runtime.h>

class FixedFunctionExcitationPatternModel
{
public:
	const SILM_Modulation* modulation; // Each vector is [ kx, ky, phase ]
	int2 roiPos;
	int numep;
	float sigma;

	PLL_DEVHOST int NumPatterns() const { return numep; }

	PLL_DEVHOST FixedFunctionExcitationPatternModel(int numep, float sigma, const SILM_Modulation* modulation, int2 roiPos)
		: numep(numep), sigma(sigma), modulation(modulation), roiPos(roiPos) {}

	PLL_DEVHOST float BackgroundPattern(int e, int x, int y) const
	{
//		SILM_Modulation mod = modulation[e];
		// this seems to match reality the most (tiny variations in bg per pattern, not as much as mod.power)
		return 1.0f / numep; 
		//		return mod.power;
	}
	PLL_DEVHOST void ExcitationPattern(float& Q, float& dQdx, float& dQdy, int e, float2 xy) const {
		// compute Q, dQ/dx, dQ/dy
		SILM_Modulation mod = modulation[e];
		float A = mod.power;
		float w = mod.kx * (xy.x + roiPos.x) + mod.ky * (xy.y + roiPos.y);
		// Todo: use phaseCoeff in the derivatives
		float ang = w - mod.phase;
		Q = A * (1.0f + mod.depth * sin(ang));
		dQdx = A * mod.kx * mod.depth * cos(ang);
		dQdy = A * mod.ky * mod.depth * cos(ang);
	}
	PLL_DEVHOST float MeanBackgroundCoefficient() const
	{
		return 1.0f / numep;
	}
};
