#pragma once

#include "DLLMacros.h"
#include "Vector.h"
#include "Estimation.h"

template<typename TResult>
class ImageProcessingQueue;

class PSF;
class sCMOS_CalibrationTransform;

typedef Vector4f Gauss2D_Theta;
typedef FisherMatrixType<Gauss2D_Theta>::type Gauss2D_FisherMatrix;
typedef EstimationResult<Gauss2D_Theta> Gauss2D_EstimationResult;

CDLL_EXPORT void Gauss2D_ComputeMLE(const float *imageData, Gauss2D_EstimationResult * results, int numspots,
	const Gauss2D_Theta* initialValues, float sigma, int imgw, int maxIterations, float startLambdaStep, bool cuda,
	Gauss2D_Theta* optimizerTrace=0, int traceBufferLength=0);

CDLL_EXPORT void Gauss2D_ComputeMLE_NewtonRaphson(const float *imageData, Gauss2D_EstimationResult* results, int numspots,
	const Vector4f* initialValues, float sigma, int imgw, int maxIterations, bool cuda,
	Vector4f* traces, int traceBufferLength);
	
CDLL_EXPORT void Gauss2D_ComputeFisherMatrix(float* expectedValue, Gauss2D_FisherMatrix* fiMatrix,
	const Gauss2D_Theta* theta, int numspots, float sigma, int imgw);

// spotList [ x y sigmaX sigmaY intensity ]
CDLL_EXPORT void Gauss2D_Draw(double* image, int imgw, int imgh, float* spotList, int nspots, float addSigma=0.0f);

typedef ImageProcessingQueue<Gauss2D_EstimationResult> Gauss2DProcessingQueue;

class ISpotDetectorFactory;
struct ImageQueueConfig;
CDLL_EXPORT Gauss2DProcessingQueue* Gauss2D_CreateImageQueue(const ImageQueueConfig& config, float sigma, int imgw, 
	int maxSpotsPerFrame, ISpotDetectorFactory* spotDetectorFactory, sCMOS_CalibrationTransform **p_calibTransform);



struct Gauss3D_Calibration
{
	Gauss3D_Calibration() {}
//	float sigma;
	float x[4];
	float y[4];
};

CDLL_EXPORT PSF* Gauss_CreatePSF_XYIBg(int roisize, float sigma, bool cuda);
CDLL_EXPORT PSF* Gauss_CreatePSF_XYIBgSigma(int roisize, bool cuda);
CDLL_EXPORT PSF* Gauss_CreatePSF_XYZIBg(int roisize, const Gauss3D_Calibration& calib, bool cuda);

// Fit X,Y,intensity, and with a fixed background supplied as constant
CDLL_EXPORT PSF* Gauss_CreatePSF_XYI(int roisize, float sigma, bool cuda);

