#pragma once

#include "DLLMacros.h"
#include "SILM_Structs.h"

#ifdef __cplusplus

#include <vector>
#include "Vector.h"
#include "MathUtils.h"
#include "Estimation.h"
#include "PSFModels/GaussianPSF.h"

template<typename TResult>
class ImageProcessingQueue;


struct SILM_EstimationResult
{
	typedef SILM_Theta Theta;
	enum { MaxSwitchFrames=2 };

	template<typename TLevMarResult>
	PLL_DEVHOST SILM_EstimationResult(Theta initialValue, TLevMarResult levMarResult, const Vector<float,16>& cov, const Vector<float,8>& crlb, int nsf) :
		estimate(levMarResult.estimate), initialValue(initialValue), crlb(crlb),
		iterations(levMarResult.iterations) {	}
	PLL_DEVHOST SILM_EstimationResult() { }

	Theta estimate;
	Theta initialValue;
	Vector<float, 4 + MaxSwitchFrames> crlb; // [X,Y,I,bg, SwitchFrameIntensities...]
	int iterations;
	uint64_t switchFrameMask, silmFrameMask;
	Int2 roiPosition;
};



#define SILM_MLE_CUDA 1				
#define SILM_MLE_FIXEDW 4

/*

imageData: float[imgw*imgw*epps*numspots]
backgroundMatrix: float[imgw*imgw*epps*numspots]
phi: float[epps*numspots]

*/
struct ImageQueueConfig;
class ISpotDetectorFactory;

// Every SILM fit takes the last iqc.historyLength frames for fitting
CDLL_EXPORT ImageProcessingQueue<Gauss2D_EstimationResult>* SILM_CreateImageQueue(const SILM_ASW_Params & p, const ImageQueueConfig & iqc,
	ISpotDetectorFactory* spotDetection, int maxSpotsPerFrame, float silmMinIntensity, const SILM_Modulation*modulation);

//
// Computes expected values for each pixel and fisher matrix
// fiMatrix: float[16 * numspots]
// expectedValue: float[imgw*imgw*nframes*numspots]
CDLL_EXPORT void SILM_ASW_ComputeFisherMatrix(float * expectedValue, Gauss2D_FisherMatrix* fiMatrix, const SILM_Modulation* modulation,
	const SILM_Theta * theta, int numspots, int nframes, const int* startPatterns, const SILM_ASW_Params& p);


CDLL_EXPORT void SILM_ASW_ComputeMLE(const float * imageData, const SILM_Modulation*modulation,
	Gauss2D_EstimationResult *results, int numspots, int nframes, const int *startPatterns, const SILM_ASW_Params& p,
	const SILM_Theta* initialTheta, const int2* roipos, int flags,
	SILM_Theta* optimizerTrace=0, int traceBufferLength=0);


/* SILM pipeline in pieces...
- Gaussian fit
- Estimate intensities/bg per frame
- Estimate initial SILM theta
- HMM
- SILM fit (with specific frames)
*/

//CDLL_EXPORT void Gauss2D_EstimateIntensityBg(const float* imageData, Vector2f *IBg, Vector2f* IBg_crlb,
//	int numspots, const Vector2f* xy, float sigma, int imgw, int maxIterations, bool cuda)


CDLL_EXPORT void SILM_ProjectPointData(const Vector3f *xyI, int numpts, int projectionWidth,
	float scale, int numProjAngles, const float *projectionAngles, float* output ,float* shifts);


CDLL_EXPORT void SILM_SignalCorrelation(const float* signalsA, int numsigA,
	const float* signalsB, int numsigB, int siglen, float *output, int useConvolution);

CDLL_EXPORT void SILM_CorrelatePhaseAndFreq(const Vector2f* pts, int numpts,
	const float* freq_list, int numfreq, const float* phase_list, int numphases, float* output);


#endif