#include "SILM.h"
#include "Gauss2DMLE.h"
#include "SMLM_MatlabAPI.h"

CDLL_EXPORT void MATLAB_SILM_ComputeExpectedValue(float * expectedValue, float * fiMatrix, const float * phi, float * theta, int numspots, 
	int nframes, int startPattern, const int * frameMask, const SILM_ASW_Params* silmParams)
{
	SILM_ASW_ComputeFisherMatrix(expectedValue, (Vector<float, 16>*)fiMatrix, phi, (SILM_Theta*)theta, numspots, nframes, startPattern, frameMask, *silmParams);
}

CDLL_EXPORT void MATLAB_SILM_BlinkHMM_ComputeMLE(const float * frameData, const float * phi, float * gaussTheta, 
	float * silmTheta, float * crlbs, int numspots, int numframes, int startPattern, 
	const SILM_ASW_Params * silmParams, bool useCuda, float * intensities, float * backgrounds, 
	float * p_on_estim, int* blinkstate, const SILM_BlinkHMM_Params * blinkParams)
{
	std::vector<SILM_EstimationResult> results(numspots);

	SILM_BlinkHMM_ComputeMLE(frameData, phi, (Vector4f*)gaussTheta, &results[0], numspots, numframes, 
		startPattern, *silmParams, useCuda, 0, 0, intensities, backgrounds, p_on_estim, blinkstate, *blinkParams);

	for (int i = 0; i < numspots; i++) {
		auto& r = results[i];
		
		((Vector6f*)crlbs)[i] = r.crlb;
		((Vector4f*)silmTheta)[i] = r.estimate;
	}
}

CDLL_EXPORT void MATLAB_Gauss2D_ComputeMLE(const float * imageData, float * gaussTheta, float * fisherMatrices, int numspots, float sigma, int imgw, int maxIterations, float startLambdaStep, bool cuda)
{
	std::vector<Gauss2D_EstimationResult> results(numspots);
	Gauss2D_ComputeMLE(imageData, &results[0], numspots, 0, sigma, imgw, maxIterations, startLambdaStep, cuda);
	for (int i = 0; i < numspots; i++)
		((Gauss2D_Theta*)gaussTheta)[i] = results[i].estimate;
}


CDLL_EXPORT void MATLAB_Gauss2D_ComputeExpectedValue(float * expectedValue, float * fisherMatrices, 
	const float * theta, int numspots, float sigma, int imgw)
{
	Gauss2D_ComputeFisherMatrix(expectedValue, (Vector<float, 16>*)fisherMatrices, (const Gauss2D_Theta*)theta, numspots, sigma, imgw);
}

