/*
Spot detection based on "Probability-based particle detection that enables threshold-free and robust in vivo single-molecule tracking" 
Using Generalized Likelihood Ratio Test statistic
*/
#pragma once


#include "ImageProcessingQueue.h"
#include "SpotDetector.h"

class GLRTSpotDetector : public IComputeTask, public ISpotLocationListSource
{
public:
	GLRTSpotDetector(int nStreams, IImageSource* src, float psfSigma, int maxSpotImages, float falsePositiveRateTarget);
	~GLRTSpotDetector();

	bool Execute(IComputeTaskManager* tm, int streamIndex);
	bool Completed(IComputeTaskManager* tm, int streamIndex);

	const int* GetSpotIndexListDevicePtr(int streamIndex, int& numIndices);

protected:
	float psfSigma, fdrTarget;
	int roisize;
	int maxSpotImages;
	IImageSource* src;

	int2 offset, croppedSize;
	DeviceImage<float> testStatistic; // [croppedsize]

	struct StreamData
	{
		int numspots;
		DeviceVector<float> spotImages;
	};

	std::vector<StreamData> streamData;
};
